import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { MatSnackBar } from '@angular/material';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { takeUntil } from 'rxjs/operators';
import { Observable } from 'rxjs/Rx';
import { environment } from '../../../../environments/environment';
import { SnackBarNotificationComponent } from '../../../library/notification/snack-bar/snack-bar-notification.component';
import { BaseService } from '../../common/base/base.service';
import { Credential } from '../../model/auth/credential';
import { LoginInfo } from '../../model/auth/login-info';
import { Principal } from '../../model/auth/Principal';
import { UiNotificationService, SnackBarNotificationLevel } from '../notification/ui-notification-service';


@Injectable()
export class AuthService extends BaseService {
	private actionUrl: string;
	private headers: HttpHeaders;

	constructor(
		private http: HttpClient,
		private snackBar: MatSnackBar,
		private language: TranslateService,
		private router: Router,
		private uiNotificationService: UiNotificationService
	) {
		super();
		this.actionUrl = environment.Server + 'auth/';

		this.headers = new HttpHeaders();
		this.headers = this.headers.set('Content-Type', 'application/json');
		this.headers = this.headers.set('Accept', 'application/json');
	}

	private clear(): void {
		localStorage.removeItem('principal');
	}

	public current(principal?: Principal): Principal {
		if (principal) {
			localStorage.setItem('principal', JSON.stringify(principal));
			return principal;
		}
		const principalJson = localStorage.getItem('principal');
		if (!principalJson) { return null; }
		const principalObj = JSON.parse(principalJson) as Principal;
		return principalObj;
	}

	//public login(credential: Credential): Observable<Principal> {
	//    const url = this.actionUrl + 'login';

	//    return this.http.post(url, credential, { headers: this.headers })
	//        .map((res: Response) => {
	//            let principal = this.current(new JsonSerializer<Principal>().fromJSONObject(res, Principal));
	//            //this.loginContextSubject.next(true);
	//            return principal;
	//        })
	//        .catch((error: any) => {
	//            //this.loginContextSubject.next(false);
	//            return Observable.throw(error);
	//        });
	//}

	public login(loginInfo: LoginInfo): Observable<Principal> {
		const url = this.actionUrl + 'externallogin';

		return this.http.post(url, loginInfo, { headers: this.headers })
			.map((res: any) => {
				const principal = this.current(res.payload);
				//this.loginContextSubject.next(true);
				return principal;
			})
			.catch((error: any) => {
				//this.loginContextSubject.next(false);
				return Observable.throw(error);
			});
	}

	public nativeLogin(credentials: Credential): Observable<Principal> {
		const url = this.actionUrl + 'nativelogin';

		return this.http.post(url, credentials, { headers: this.headers })
			.map((res: any) => {
				const principal = this.current(res.payload);
				//this.loginContextSubject.next(true);
				return principal;
			})
			.catch((error: any) => {
				//this.loginContextSubject.next(false);
				return Observable.throw(error);
			});
	}


	public logout(): void {
		const url = this.actionUrl + 'logout';
		const principal = this.current();
		this.clear();

		if (!principal) { return; }
		let headers = this.headers;
		headers = headers.set('AuthToken', principal.token);
		this.http.post(url, null, { headers: headers })
			.pipe(takeUntil(this._destroyed))
			.subscribe(
				res => this.onLogOutSuccess(res),
				error => this.onLogOutError(error)
			);
	}

	public me(): Observable<Principal> {
		const url = this.actionUrl + '/me';
		const principal = this.current();
		if (!principal) {
			this.clear();
			return Observable.of<Principal>();
		}
		let headers = this.headers;
		headers = headers.set('AuthToken', principal.token);
		return this.http.post(url, null, { headers: headers })
			.map((res: any) => {
				const princ = this.current(res.payload);
				return princ;
			})
			.catch((error: any) => {
				//console.warn('could not retrieve me info:\n', error);
				this.clear();
				const princ = this.current();
				this.router.navigate(['/login']);
				return Observable.of<Principal>(princ);
			});
	}

	public onLogOutSuccess(logoutMessage: any) {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-LOGOUT'), SnackBarNotificationLevel.Success);
		this.router.navigate(['/login']);
	}

	public onLogOutError(errorMessage: string) {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.UNSUCCESSFUL-LOGOUT'), SnackBarNotificationLevel.Error);
		this.router.navigate(['/login']);
	}
}
