import { DataSource } from '@angular/cdk/table';
import { Component, OnInit, ViewChild } from '@angular/core';
import { MatPaginator, MatSnackBar, MatSort, PageEvent } from '@angular/material';
import { ActivatedRoute, Params, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { Observable } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { BaseComponent } from '../../../../core/common/base/base.component';
import { DmpProfileListing } from '../../../../core/model/dmp-profile/dmp-profile-listing';
import { DmpProfileCriteria } from '../../../../core/query/dmp/dmp-profile-criteria';
import { DmpProfileService } from '../../../../core/services/dmp/dmp-profile.service';
import { DataTableRequest } from '../../../../core/model/data-table/data-table-request';
import { DmpProfileCriteriaComponent } from './criteria/dmp-profile-criteria.component';
import { BreadcrumbItem } from '../../../misc/breadcrumb/definition/breadcrumb-item';

@Component({
	selector: 'app-dmp-profile-listing-component',
	templateUrl: 'dmp-profile-listing.component.html',
	styleUrls: ['./dmp-profile-listing.component.scss']
})
export class DmpProfileListingComponent extends BaseComponent implements OnInit {

	@ViewChild(MatPaginator) _paginator: MatPaginator;
	@ViewChild(MatSort) sort: MatSort;
	@ViewChild(DmpProfileCriteriaComponent) criteria: DmpProfileCriteriaComponent;

	dataSource: DatasetDataSource | null;
	displayedColumns: String[] = ['label', 'status', 'created'];
	pageEvent: PageEvent;
	titlePrefix: String;
	dmpId: String;
	breadCrumbs: Observable<BreadcrumbItem[]>;

	statuses = [
		{ value: '0', viewValue: 'Active' },
		{ value: '1', viewValue: 'Inactive' }
	];

	constructor(
		private router: Router,
		private languageService: TranslateService,
		public snackBar: MatSnackBar,
		public route: ActivatedRoute,
		public dmpProfileService: DmpProfileService
	) {
		super();
	}

	ngOnInit() {
		this.route.params
			.pipe(takeUntil(this._destroyed))
			.subscribe((params: Params) => {
				this.dmpId = params['dmpId'];
				this.criteria.setCriteria(this.getDefaultCriteria());
				this.refresh();
				this.criteria.setRefreshCallback(() => this.refresh());
				this.breadCrumbs = Observable.of([{
					parentComponentName: null,
					label: this.languageService.instant('NAV-BAR.DMP-TEMPLATES'),
					url: '/dmp-profiles'
				}]);
			});
	}

	refresh() {
		this.dataSource = new DatasetDataSource(this.dmpProfileService, this._paginator, this.sort, this.criteria);
	}

	rowClick(rowId: String) {
		this.router.navigate(['dmp-profiles/' + rowId]);
	}

	getDefaultCriteria(): DmpProfileCriteria {
		const defaultCriteria = new DmpProfileCriteria();
		return defaultCriteria;
	}

	// makeItPublic(id: String) {
	//   debugger;
	//   this.datasetService.makeDatasetPublic(id).pipe(takeUntil(this._destroyed)).subscribe();
	// }
}

export class DatasetDataSource extends DataSource<DmpProfileListing> {

	totalCount = 0;

	constructor(
		private _service: DmpProfileService,
		private _paginator: MatPaginator,
		private _sort: MatSort,
		private _criteria: DmpProfileCriteriaComponent
	) {
		super();

	}

	connect(): Observable<DmpProfileListing[]> {
		const displayDataChanges = [
			this._paginator.page
			//this._sort.matSortChange
		];

		return Observable.merge(...displayDataChanges)
			.startWith(null)
			.switchMap(() => {
				const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
				let fields: Array<string> = new Array();
				if (this._sort.active) { fields = this._sort.direction === 'asc' ? ['+' + this._sort.active] : ['-' + this._sort.active]; }
				const request = new DataTableRequest<DmpProfileCriteria>(startIndex, this._paginator.pageSize, { fields: fields });
				request.criteria = this._criteria.criteria;
				return this._service.getPaged(request);
			})
			/*.catch((error: any) => {
              this._snackBar.openFromComponent(SnackBarNotificationComponent, {
                data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
                duration: 3000,
                extraClasses: ['snackbar-warning']
              });
              //this._criteria.criteria.onCallbackError(error);
              return Observable.of(null);
            })*/
			.map(result => {
				return result;
			})
			.map(result => {
				if (!result) { return []; }
				if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
				return result.data;
			});
	}

	disconnect() {
		// No-op
	}
}
