import { Component, OnInit } from "@angular/core";
import { RecentActivityType } from "../../../core/common/enum/recent-activity-type";
import { BaseComponent } from "../../../core/common/base/base.component";
import { Router } from "@angular/router";
import { AuthService } from "../../../core/services/auth/auth.service";
import { UserService } from "../../../core/services/user/user.service";
import { takeUntil } from "rxjs/operators";
import { DmpService } from "../../../core/services/dmp/dmp.service";
import { DataTableRequest } from "../../../core/model/data-table/data-table-request";
import { DmpCriteria } from "../../../core/query/dmp/dmp-criteria";
import { DmpListingModel } from "../../../core/model/dmp/dmp-listing";
import { EnumUtils } from "../../../core/services/utilities/enum-utils.service";

@Component({
	selector: "app-recent-visited-activity",
	templateUrl: "./recent-visited-activity.component.html",
	styleUrls: ["./recent-visited-activity.component.css"]
})
export class RecentVisitedActivityComponent extends BaseComponent
	implements OnInit {
	dmpActivities: DmpListingModel[];
	recentActivityItems: any[];
	recentActivityTypeEnum = RecentActivityType;
	public: boolean = false;

	constructor(
		private router: Router,
		private authentication: AuthService,
		private dmpService: DmpService,
		public enumUtils: EnumUtils
	) {
		super();
	}

	ngOnInit() {
		if (this.isAuthenticated()) {
			const fields: Array<string> = ["-created"];
			const dmpDataTableRequest: DataTableRequest<DmpCriteria> = new DataTableRequest(0, null, { fields: fields });
			dmpDataTableRequest.criteria = new DmpCriteria();
			dmpDataTableRequest.criteria.like = "";
			this.dmpService
				.getPaged(dmpDataTableRequest, "listing")
				.subscribe(response => {
					this.dmpActivities = response.data;
				});
		}
	}

	redirect(id: string, type: RecentActivityType) {
		switch (type) {
			case RecentActivityType.Grant: {
				this.router.navigate(["grants/edit/" + id]);
				return;
			}
			case RecentActivityType.Dataset: {
				this.router.navigate(["datasets/edit/" + id]);
				return;
			}
			case RecentActivityType.Dmp: {
				this.router.navigate(["plans/edit/" + id]);
				return;
			}
			default:
				throw new Error("Unsupported Activity Type ");
		}
	}

	public isAuthenticated(): boolean {
		return !!this.authentication.current();
	}

	navigateToUrl() { }
}
