import { Component, OnInit, Input } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { SingleAutoCompleteConfiguration } from '../../../../library/auto-complete/single/single-auto-complete-configuration';
import { RequestItem } from '../../../../core/query/request-item';
import { GrantCriteria } from '../../../../core/query/grant/grant-criteria';
import { GrantService } from '../../../../core/services/grant/grant.service';
import { LanguageResolverService } from '../../../../services/language-resolver/language-resolver.service';
import { GrantTabModel } from './grant-tab-model';

@Component({
	selector: 'app-grant-tab',
	templateUrl: './grant-tab.component.html',
	styleUrls: ['./grant-tab.component.scss']
})
export class GrantTabComponent implements OnInit {

	@Input() formGroup: FormGroup;
	@Input() isNew: boolean;
	@Input() isFinalized: boolean;
	isCreateNew = false;
	grant: GrantTabModel;

	grantAutoCompleteConfiguration: SingleAutoCompleteConfiguration;

	constructor(
		private grantService: GrantService,
		public languageResolverService: LanguageResolverService
	) { }

	ngOnInit() {

		const grantRequestItem: RequestItem<GrantCriteria> = new RequestItem();
		grantRequestItem.criteria = new GrantCriteria();

		this.grantAutoCompleteConfiguration = {
			filterFn: this.searchGrant.bind(this),
			initialItems: (extraData) => this.searchGrant(''),
			displayFn: (item) => item['label'],
			titleFn: (item) => item['label']
		};

		this.isCreateNew = (this.formGroup.get('label').value != null && this.formGroup.get('label').value.length > 0);
		this.setValidators();
	}

	searchGrant(query: string) {
		const grantRequestItem: RequestItem<GrantCriteria> = new RequestItem();
		grantRequestItem.criteria = new GrantCriteria();
		grantRequestItem.criteria.like = query;
		return this.grantService.getWithExternal(grantRequestItem);
	}

	create() {
		this.isCreateNew = !this.isCreateNew;
		this.setValidators();
	}

	setValidators() {
		if (this.isCreateNew) {
			this.formGroup.get('existGrant').disable();
			this.formGroup.get('label').enable();
			this.formGroup.get('description').enable();
		} else if (this.isFinalized) {
			this.formGroup.get('existGrant').disable();
			this.formGroup.get('label').disable();
			this.formGroup.get('description').disable();
		}
		else {
			this.formGroup.get('existGrant').enable();
			this.formGroup.get('label').disable();
			this.formGroup.get('label').reset();
			this.formGroup.get('description').disable();
			this.formGroup.get('description').reset();
		}
	}

}
